#include "main.h"
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <bcm2835.h>
#include <stdbool.h>
#include <stdlib.h>
#include <unistd.h>

#include <stdint.h>

#define LEDG 	RPI_V2_GPIO_P1_13
#define LEDR 	RPI_V2_GPIO_P1_15
#define SPI_CLOCK_DIVIDER_26	26

#define MESSAGE_BUF_SIZE	66
#define MAX_FIFO_SIZE		66

#define MSG_REMAINING_LEN 0

int main( int argc, char **argv){


	setbuf (stdout, NULL);
	printf ("Legacy OOK Transmit Program\n");

	if (!bcm2835_init())
		return 1;
	
	int cmdno;
	int groupno;
	int onoff;

	if ( argc != 3 ) // argc should be 2 for correct execution
	{
		printf( "usage: %s [socket number (1-40)] [on|off (1|0)]\n", argv[0] );
		return 0;
	}
	else
	{
		//cmdno = atoi(argv[2]);
		//groupno = atoi(argv[1]);
		groupno = (atoi(argv[1])-1)*0.25;
		cmdno = (atoi(argv[1])-(groupno*4)-1)*2;
		onoff = atoi(argv[2]);
	}
	{	
		switch (onoff) {
		case 1 :
		cmdno = cmdno;
		break;
		case 0 :
		cmdno = cmdno + 1;
		break;
		default :
		cmdno = cmdno + 1;
		}
	}

	// LED INIT
	bcm2835_gpio_fsel(LEDG, BCM2835_GPIO_FSEL_OUTP);			// LED green
	bcm2835_gpio_fsel(LEDR, BCM2835_GPIO_FSEL_OUTP);			// LED red
	bcm2835_gpio_write(LEDG, LOW);
	bcm2835_gpio_write(LEDR, LOW);
	// SPI INIT
	bcm2835_spi_begin();	
	bcm2835_spi_setClockDivider(SPI_CLOCK_DIVIDER_26); 			// 250MHz / 26 = 9.6MHz
	bcm2835_spi_setDataMode(BCM2835_SPI_MODE0); 				// CPOL = 0, CPHA = 0
	bcm2835_spi_chipSelect(BCM2835_SPI_CS1);					// chip select 1

	// Set the trasncevier up for FSK, 433.92MHz compatible with 1527 chip
	HRF_config_OOK();
	
	// wait until ready after mode switching
	HRF_wait_for(ADDR_IRQFLAGS1, MASK_MODEREADY, true);			
	HRF_clr_fifo();

	
	uint8_t i;
	while (i < 1) {
		// send message blocks
			for (i=0; i< 1;)
			{
				{				
					// printf("\nSend OOK message: [%d]\t", cmdno);
					printf("\nSend OOK message Socket %d %d\n", atoi(argv[1]), onoff);
					HRF_send_OOK_msg(cmdno, groupno); 
					i++;
				}
		 }	
	}
	bcm2835_spi_end();
	return 0;
}

// Set up the transceiver
void HRF_config_OOK(){
	static regSet_t regSetup[] = {
		{ADDR_REGDATAMODUL, VAL_REGDATAMODUL_OOK},	// modulation scheme OOK
		{ADDR_FDEVMSB, 		0}, 					// frequency deviation -> 0kHz 
		{ADDR_FDEVLSB, 		0}, 					// frequency deviation -> 0kHz
		{ADDR_FRMSB, 		VAL_FRMSB433},			// carrier freq -> 433.92MHz 0x6C7AE1
		{ADDR_FRMID, 		VAL_FRMID433},			// carrier freq -> 433.92MHz 0x6C7AE1
		{ADDR_FRLSB, 		VAL_FRLSB433},			// carrier freq -> 433.92MHz 0x6C7AE1
		{ADDR_RXBW, 		VAL_RXBW120},			// channel filter bandwidth 120kHz
		{ADDR_PREAMBLELSB, 	0},						// preamble size LSB 0
		{ADDR_SYNCCONFIG, 	VAL_SYNCCONFIG4},		// Size of the Synch word = 4 (SyncSize + 1)
		{ADDR_SYNCVALUE1, 	0x80},					// sync value 1
		{ADDR_SYNCVALUE2, 	0},						// sync value 2
		{ADDR_SYNCVALUE3, 	0},						// sync value 3
		{ADDR_SYNCVALUE4, 	0},						// sync value 4
		{ADDR_PACKETCONFIG1, VAL_PACKETCONFIG1OOK},	// Fixed length, no Manchester coding
		{ADDR_PAYLOADLEN, 	VAL_PAYLOADLEN_OOK},	// Payload Length
		{ADDR_FIFOTHRESH, 	VAL_FIFOTHRESH30},		// Condition to start packet transmission: wait for 30 bytes in FIFO
		{ADDR_OPMODE, 		MODE_TRANSMITER}		// Transmitter mode
	}; 
	uint8_t size = sizeof(regSetup)/sizeof(regSet_t), i;
	for (i=0; i<size; ++i){
		HRF_reg_W(regSetup[i].addr, regSetup[i].val);
	}
}

void HRF_send_OOK_msg(uint8_t relayState, int groupCode)
{
	uint8_t buf[17];
	uint8_t i;

	HRF_config_OOK();
	
	// Sync words pattern as per datasheet for 1527
	buf[1] = 0x80;				
	buf[2] = 0x00;
	buf[3] = 0x00;
	buf[4] = 0x00;
	
	// Set up the 20 bit address field
	switch (groupCode)
	{
	case 0 :
	buf[5] = 136;
	buf[6] = 136;
	buf[7] = 232;
	buf[8] = 238;
	buf[9] = 142;
	buf[10] = 238;
	buf[11] = 232;
	buf[12] = 232;
	buf[13] = 238;
	buf[14] = 142;
	break;
	case 1 :
        buf[5] = 232;
        buf[6] = 232;
        buf[7] = 136;
        buf[8] = 232;
        buf[9] = 136;
        buf[10] = 232;
        buf[11] = 142;
        buf[12] = 142;
        buf[13] = 232;
        buf[14] = 232;
	break;
	case 2 :
        buf[5] = 238;
        buf[6] = 142;
        buf[7] = 238;
        buf[8] = 136;
        buf[9] = 136;
        buf[10] = 142;
        buf[11] = 142;
        buf[12] = 136;
        buf[13] = 238;
        buf[14] = 142;
	break;
	case 3 :
	buf[5] = 136;
	buf[6] = 238;
	buf[7] = 142;
	buf[8] = 136;
	buf[9] = 142;
	buf[10] = 142;
	buf[11] = 136;
	buf[12] = 238;
	buf[13] = 136;
	buf[14] = 142;
	break;
	case 4 :
	buf[5] = 142;
	buf[6] = 142;
	buf[7] = 136;
	buf[8] = 232;
	buf[9] = 238;
	buf[10] = 238;
	buf[11] = 232;
	buf[12] = 136;
	buf[13] = 238;
	buf[14] = 238;
	break;
	case 5 :
	buf[5] = 232;
	buf[6] = 142;
	buf[7] = 142;
	buf[8] = 142;
	buf[9] = 238;
	buf[10] = 142;
	buf[11] = 238;
	buf[12] = 142;
	buf[13] = 136;
	buf[14] = 142;
	break;
	case 6 :
	buf[5] = 232;
	buf[6] = 142;
	buf[7] = 238;
	buf[8] = 238;
	buf[9] = 142;
	buf[10] = 136;
	buf[11] = 136;
	buf[12] = 142;
	buf[13] = 142;
	buf[14] = 136;
	break;
	case 7 :
	buf[5] = 136;
	buf[6] = 232;
	buf[7] = 142;
	buf[8] = 238;
	buf[9] = 238;
	buf[10] = 136;
	buf[11] = 142;
	buf[12] = 136;
	buf[13] = 136;
	buf[14] = 142;
	break;
	case 8 :
	buf[5] = 238;
	buf[6] = 238;
	buf[7] = 142;
	buf[8] = 136;
	buf[9] = 238;
	buf[10] = 142;
	buf[11] = 136;
	buf[12] = 136;
	buf[13] = 232;
	buf[14] = 232;
	break;
	case 9 :
	buf[5] = 238;
	buf[6] = 232;
	buf[7] = 238;
	buf[8] = 136;
	buf[9] = 232;
	buf[10] = 142;
	buf[11] = 136;
	buf[12] = 142;
	buf[13] = 238;
	buf[14] = 238;
	break;
	default :
        buf[5] = 136;
        buf[6] = 136;
        buf[7] = 232;
        buf[8] = 238;
        buf[9] = 142;
        buf[10] = 238;
        buf[11] = 232;
        buf[12] = 232;
        buf[13] = 238;
        buf[14] = 142;
	}
	
	// switch sockets on and off using D0-D3 codes
	switch (relayState)
	{
		case 0 :
//			printf("Socket 1 ON\n");
			buf[15] = 0xEE;		// D0-high, D1-h		// S1 on
			buf[16] = 0xEE;		// D2-h, D3-h
			break;

		case 1 :

//			printf("Socket 1 OFF\n");
			buf[15] = 0xEE;		// D0-high, D1-h		// S1 off
			buf[16] = 0xE8;		// D2-h, D3-l
			break;

		case 2 :
//			printf("Socket 2 ON\n");
			buf[15] = 0x8E;		// D0-l, D1-h			// S2 on
			buf[16] = 0xEE;		// D2-h, D3-h
			break;

		case 3 :
//			printf("Socket 2 OFF\n");
			buf[15] = 0x8E;		// D0-l, D1-h			// S2 off
			buf[16] = 0xE8;		// D2-h, D3-l
			break;

		case 4 :
//			printf("Socket 3 ON\n");
			buf[15] = 0xE8;		// D0-high, D1-l		// S3 on
			buf[16] = 0xEE;		// D2-h, D3-h
			break;

		case 5 :
//			printf("Socket 3 OFF\n");
			buf[15] = 0xE8;		// D0-high, D1-l		// S3 off
			buf[16] = 0xE8;		// D2-h, D3-l
			break;

		case 6 :
//			printf("Socket 4 ON\n");
			buf[15] = 0x88;		// D0-l, D1-l		// S4 on
			buf[16] = 0xEE;		// D2-h, D3-h
			break;

		case 7 :
//			printf("Socket 4 OFF\n");
			buf[15] = 0x88;		// D0-l, D1-l		// S4 off
			buf[16] = 0xE8;		// D2-h, D3-l
			break;

		case 8 :
			printf("ALL ON\n");
			buf[15] = 0xEE;		// D0-high, D1-h		// all on
			buf[16] = 0x8E;		// D2-l, D3-h
			break;

		case 9 :
			printf("ALL OFF\n");
			buf[15] = 0xEE;		// D0-high, D1-h		// all off
			buf[16] = 0x88;		// D2-l, D3-l
			break;
	}

	// wait for ModeReady + TX ready
	HRF_wait_for (ADDR_IRQFLAGS1, MASK_MODEREADY | MASK_TXREADY, TRUE);
	
	// Turn the red led on 	
	bcm2835_gpio_write(RPI_V2_GPIO_P1_15, 1);
	
	// Send few more same messages				
	HRF_reg_Wn(buf + 4, 0, 12);					

	for (i = 0; i < 32; ++i)
	{
		HRF_wait_for(ADDR_IRQFLAGS2, MASK_FIFOLEVEL, FALSE);
		HRF_reg_Wn(buf, 0, 16);			// +4 sync bytes
	}

	// wait for Packet sent
	HRF_wait_for (ADDR_IRQFLAGS2, MASK_PACKETSENT, TRUE);		
	
	// Turn the led off 
	bcm2835_gpio_write(RPI_V2_GPIO_P1_15, 0);	
					
	HRF_assert_reg_val(ADDR_IRQFLAGS2, MASK_FIFONOTEMPTY | MASK_FIFOOVERRUN, FALSE, "are all bytes sent?");
}

void HRF_clr_fifo(void){
	
	// FIFO FLAG FifoNotEmpty
	while (HRF_reg_R(ADDR_IRQFLAGS2) & MASK_FIFONOTEMPTY)				
	{
		HRF_reg_R(ADDR_FIFO);
	}
	return;
}
void HRF_reg_Rn(uint8_t *retBuf, uint8_t addr, uint8_t size){
	uint8_t tmp = retBuf[0];
	retBuf[0] = addr;
	bcm2835_spi_transfern((char*)retBuf, size + 1);
	retBuf[0] = tmp;
	return;
}
void HRF_reg_Wn(uint8_t *retBuf, uint8_t addr, uint8_t size){
	uint8_t tmp = retBuf[0];
	retBuf[0] = addr | MASK_WRITE_DATA;
	bcm2835_spi_writenb((char*)retBuf, size + 1);
	retBuf[0] = tmp;
	return;
}
uint8_t HRF_reg_R(uint8_t addr){
	uint8_t buf[2];
	buf[0] = addr;
	bcm2835_spi_transfern((char*)buf, 2);
	return buf[1];
}
void HRF_reg_W(uint8_t addr, uint8_t val){
	uint8_t buf[2];
	buf[0] = addr | MASK_WRITE_DATA;
	buf[1] = val;
	bcm2835_spi_writenb((char*)buf, 2);
	return;
}
void HRF_change_mode(uint8_t mode){
	uint8_t buf[2];
	buf[0] = ADDR_OPMODE | MASK_WRITE_DATA;
	buf[1] = mode;
	bcm2835_spi_writenb((char*)buf, 2);
}
void HRF_assert_reg_val(uint8_t addr, uint8_t mask, uint8_t val, char *desc){
	uint8_t buf[2];
	buf[0] = addr;
	bcm2835_spi_transfern((char*)buf, 2);
	if (val){
		if ((buf[1] & mask) != mask)
			printf("ASSERTION FAILED: addr:%02x, expVal:%02x(mask:%02x) != val:%02x, desc: %s\n", addr, val, mask, buf[1], desc);
	} else {
		if ((buf[1] & mask) != 0)
			printf("ASSERTION FAILED: addr:%02x, expVal:%02x(mask:%02x) != val:%02x, desc: %s\n", addr, val, mask, buf[1], desc);
	}
}

void HRF_wait_for(uint8_t addr, uint8_t mask, uint8_t val){

	uint8_t ret;
	do {
		ret = HRF_reg_R(addr);
	} while ((ret & mask) != (val ? mask : 0));
}
